@@ -24,7 +24,9 @@
 
 #include <glib-object.h>
 
+#include <gabble/capabilities.h>
 #include <gabble/capabilities-set.h>
+#include <gabble/caps-channel-manager.h>
 #include <gabble/caps-hash.h>
 #include <gabble/connection.h>
 #include <gabble/error.h>
