@@ -2039,16 +2039,11 @@ conn_presence_finalize (GabbleConnection *conn)
   if (priv->iq_list_push_cb != NULL)
     lm_message_handler_unref (priv->iq_list_push_cb);
 
+  g_slice_free (GabbleConnectionPresencePrivate, priv);
+
   tp_presence_mixin_finalize ((GObject *) conn);
 }
 
-
-void
-conn_presence_iface_init (gpointer g_iface, gpointer iface_data)
-{
-  tp_presence_mixin_iface_init (g_iface, iface_data);
-}
-
 static void
 conn_presence_send_directed_presence (
     GabbleSvcConnectionInterfaceGabbleDecloak *conn,
