@@ -107,6 +107,9 @@ const gchar *gabble_jingle_session_get_sid (GabbleJingleSession *sess);
 JingleDialect gabble_jingle_session_get_dialect (GabbleJingleSession *sess);
 
 gboolean gabble_jingle_session_can_modify_contents (GabbleJingleSession *sess);
+gboolean gabble_jingle_session_peer_has_quirk (
+    GabbleJingleSession *self,
+    const gchar *quirk);
 
 typedef void (*JingleReplyHandler) (GObject *, gboolean success,
     LmMessage *reply);
