@@ -10,7 +10,7 @@ from caps_helper import compute_caps_hash, \
     text_fixed_properties, text_allowed_properties, \
     stream_tube_fixed_properties, stream_tube_allowed_properties, \
     dbus_tube_fixed_properties, dbus_tube_allowed_properties, \
-    ft_fixed_properties, ft_allowed_properties
+    ft_fixed_properties, ft_allowed_properties_with_metadata
 
 import ns
 from jingleshareutils import test_ft_caps_from_contact
@@ -29,7 +29,7 @@ generic_ft_caps = [(text_fixed_properties, text_allowed_properties),
                    (stream_tube_fixed_properties, \
                         stream_tube_allowed_properties),
                    (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
-                   (ft_fixed_properties, ft_allowed_properties)]
+                   (ft_fixed_properties, ft_allowed_properties_with_metadata)]
 
 generic_caps = [(text_fixed_properties, text_allowed_properties),
                    (stream_tube_fixed_properties, \
